"use strict";
const binaryen = require("binaryen");
const webpack_sources_1 = require("webpack-sources");
const PLUGIN_NAME = "OptimizeWasmPlugin";
class OptimizeWasmPlugin {
    constructor(options = {}) {
        const { test = /\.wasm$/ } = options;
        this.options = {
            test,
        };
    }
    apply(compiler) {
        compiler.hooks.compilation.tap(PLUGIN_NAME, (compilation) => {
            compilation.hooks.optimizeChunkAssets.tapAsync(PLUGIN_NAME, (chunks, callback) => {
                Array.from(chunks)
                    .reduce((acc, chunk) => acc.concat(chunk.files || []), [])
                    .concat(compilation.additionalChunkAssets || [])
                    .filter((file) => this.options.test.test(file))
                    .forEach((file) => {
                    const original = compilation.assets[file];
                    const buffer = original.source();
                    const mod = binaryen.readBinary(new Uint8Array(buffer));
                    mod.optimize();
                    if (!mod.validate())
                        throw new Error("validation error");
                    const optimized = new webpack_sources_1.SourceMapSource(mod.emitBinary().buffer, file, null, original);
                    // The current size method implementation assumes source returns a
                    // string, so we're overriding this for now to allow webpack to report
                    // a size metric for wasm assets which are represented by ArrayBuffer.
                    optimized.size = function () {
                        return this.source().byteLength;
                    };
                    compilation.assets[file] = optimized;
                });
                callback();
            });
        });
    }
}
module.exports = OptimizeWasmPlugin;
//# sourceMappingURL=plugin.js.map